# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tracker_packet.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14tracker_packet.proto\x12\x08protocol\"\x1d\n\x0eTrackerRequest\x12\x0b\n\x03uid\x18\x01 \x01(\t\"\x1f\n\x10RecordingRequest\x12\x0b\n\x03tag\x18\x01 \x01(\t\"3\n\x11RecordingResponse\x12\x0f\n\x07started\x18\x01 \x01(\x08\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"S\n\x13TrackerMetaResponse\x12\r\n\x05valid\x18\x01 \x01(\x08\x12\r\n\x05index\x18\x02 \x01(\x04\x12\x11\n\tsys_ts_ns\x18\x03 \x01(\x04\x12\x0b\n\x03uid\x18\x04 \x01(\t\"\xad\x01\n\x15TrackerSingleResponse\x12+\n\x04meta\x18\x01 \x01(\x0b\x32\x1d.protocol.TrackerMetaResponse\x12\r\n\x05pos_x\x18\x02 \x01(\x01\x12\r\n\x05pos_y\x18\x03 \x01(\x01\x12\r\n\x05pos_z\x18\x04 \x01(\x01\x12\r\n\x05rot_w\x18\x05 \x01(\x01\x12\r\n\x05rot_x\x18\x06 \x01(\x01\x12\r\n\x05rot_y\x18\x07 \x01(\x01\x12\r\n\x05rot_z\x18\x08 \x01(\x01\"u\n\x14TrackerGroupResponse\x12+\n\x04meta\x18\x01 \x01(\x0b\x32\x1d.protocol.TrackerMetaResponse\x12\x30\n\x07payload\x18\x02 \x03(\x0b\x32\x1f.protocol.TrackerSingleResponse2\xf9\x03\n\x0eTrackerService\x12M\n\x0fGetTrackerGroup\x12\x18.protocol.TrackerRequest\x1a\x1e.protocol.TrackerGroupResponse\"\x00\x12U\n\x15GetTrackerGroupStream\x12\x18.protocol.TrackerRequest\x1a\x1e.protocol.TrackerGroupResponse\"\x00\x30\x01\x12O\n\x10GetTrackerSingle\x12\x18.protocol.TrackerRequest\x1a\x1f.protocol.TrackerSingleResponse\"\x00\x12W\n\x16GetTrackerSingleStream\x12\x18.protocol.TrackerRequest\x1a\x1f.protocol.TrackerSingleResponse\"\x00\x30\x01\x12K\n\x0eStartRecording\x12\x1a.protocol.RecordingRequest\x1a\x1b.protocol.RecordingResponse\"\x00\x12J\n\rStopRecording\x12\x1a.protocol.RecordingRequest\x1a\x1b.protocol.RecordingResponse\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tracker_packet_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TRACKERREQUEST._serialized_start=34
  _TRACKERREQUEST._serialized_end=63
  _RECORDINGREQUEST._serialized_start=65
  _RECORDINGREQUEST._serialized_end=96
  _RECORDINGRESPONSE._serialized_start=98
  _RECORDINGRESPONSE._serialized_end=149
  _TRACKERMETARESPONSE._serialized_start=151
  _TRACKERMETARESPONSE._serialized_end=234
  _TRACKERSINGLERESPONSE._serialized_start=237
  _TRACKERSINGLERESPONSE._serialized_end=410
  _TRACKERGROUPRESPONSE._serialized_start=412
  _TRACKERGROUPRESPONSE._serialized_end=529
  _TRACKERSERVICE._serialized_start=532
  _TRACKERSERVICE._serialized_end=1037
# @@protoc_insertion_point(module_scope)
